//
// Created by viciu on 4/8/24.
//
#include <Arduino.h>
#include <ArduinoOTA.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include "wifi.h"
#include "DHTesp.h"

ESP8266WebServer server(80);
DHTesp dht;
float temperature = NAN;
float humidity = NAN;
unsigned long last_measure = 0;

#ifndef HOSTNAME
#define HOSTNAME "dht"
#endif

void root_page() {
    String resp = F("");

    resp = "{";
    if (humidity != NAN) {
        resp.concat("\"rh\":");
        resp.concat(String(humidity));
    }

    if (temperature != NAN) {
        if (resp.length() > 2) {
            resp.concat(",");
        }
        resp.concat("\"temp\":");
        resp.concat(String(temperature));
    }
    if (resp.length() > 2) {
        resp.concat(",");
    }
    resp.concat("\"age\":");
    resp.concat(String((millis() - last_measure) / 1000.0));
    resp.concat("}");
    Serial.println(resp);
    server.send(200, "application/json", resp);
}


void getResults() {
    temperature = dht.getTemperature();
    humidity = dht.getHumidity();
    last_measure = millis();
}

void setup() {
    Serial.begin(115200);

    WiFi.begin(SSID, PASSWORD);
    uint32_t notConnectedCounter = 0;
    Serial.print("Wifi connecting: ");
    while (WiFi.status() != WL_CONNECTED) {
        delay(300);
        Serial.print(".");
        notConnectedCounter++;
        if (notConnectedCounter > 150) { // Reset board if not connected after 5s
            Serial.println("Resetting due to Wifi not connecting...");
            ESP.restart();
        }
    }
    Serial.print("\nWifi connected, IP address: ");
    Serial.println(WiFi.localIP());

    ArduinoOTA.setHostname(HOSTNAME);
    ArduinoOTA.begin(true);

    server.begin();
    server.on("/", root_page);

    dht.setup(02, DHTesp::DHT11);
    delay(dht.getMinimumSamplingPeriod());
    getResults();

}


void loop(){
    ArduinoOTA.handle();
    server.handleClient();
    if (millis() - last_measure > 20 * 1000) {
        getResults();
    }
}